package com.fuint.common.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.fuint.common.dto.ext.TableCategoryDto;
import com.fuint.common.param.PageParam;
import com.fuint.framework.exception.BusinessCheckException;
import com.fuint.framework.pagination.PaginationRequest;
import com.fuint.framework.pagination.PaginationResponse;
import com.fuint.framework.web.PageResult;
import com.fuint.repository.bean.TableCategoryBean;
import com.fuint.repository.model.MtTableCategory;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutionException;

/**
 * 餐桌分类表业务接口
 *
 * Created by pzg
 * CopyRight https://www.fuint.cn
 */
public interface TableCategoryService extends IService<MtTableCategory> {

    /**
     * 分页查询列表
     *
     * @param paginationRequest
     * @return
     */
    PaginationResponse<MtTableCategory> queryTableCategoryListByPagination(PaginationRequest paginationRequest) throws BusinessCheckException;

    /**
     * 添加餐桌分类表
     *
     * @param  mtTableCategory
     * @throws BusinessCheckException
     * @return
     */
    MtTableCategory addTableCategory(MtTableCategory mtTableCategory) throws BusinessCheckException;

    /**
     * 根据ID获取餐桌分类表信息
     *
     * @param id ID
     * @throws BusinessCheckException
     * @return
     */
    MtTableCategory queryTableCategoryById(Long id) throws BusinessCheckException;

    /**
     * 根据ID删除餐桌分类表
     *
     * @param id ID
     * @param operator 操作人
     * @throws BusinessCheckException
     * @return
     */
    void deleteTableCategory(Long id, String operator) throws BusinessCheckException;

    /**
     * 更新餐桌分类表
     * @param  mtTableCategory
     * @throws BusinessCheckException
     * @return
     * */
    MtTableCategory updateTableCategory(MtTableCategory mtTableCategory) throws BusinessCheckException;

    /**
     * 根据条件搜索餐桌分类表
     *
     * @param params 查询参数
     * @throws BusinessCheckException
     * @return
     * */
    List<MtTableCategory> queryTableCategoryListByParams(Map<String, Object> params) throws BusinessCheckException;

    void saveHandler(TableCategoryDto tableCategoryDto) throws BusinessCheckException;

    void updateStatus(TableCategoryDto tableCategoryDto) throws BusinessCheckException;

    PageResult<TableCategoryBean> selectTableCategoryList(String categoryName, PageParam pageParam) throws ExecutionException, InterruptedException;

    void deleteHandler(Long id) throws BusinessCheckException;

    /**
     * 校验分类是否存在且有效
     * @param categoryId 分类ID
     * @return 是否存在有效分类
     */
    boolean existCategory(Long categoryId) throws BusinessCheckException;

}
