package com.fuint.common.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.Snowflake;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fuint.common.dto.AccountInfo;
import com.fuint.common.dto.MtPurchaseOrderDto;
import com.fuint.common.dto.MtPurchaseOrderItemDto;
import com.fuint.common.dto.ext.PurchaseOrderSearchDto;
import com.fuint.common.enums.StatusEnum;
import com.fuint.common.service.PurchaseOrderItemService;
import com.fuint.common.service.PurchaseOrderService;
import com.fuint.common.util.AuthUserUtil;
import com.fuint.common.util.I18nUtil;
import com.fuint.common.vo.MtPurchaseOrderItemVo;
import com.fuint.common.vo.MtPurchaseOrderVo;
import com.fuint.framework.exception.BusinessCheckException;
import com.fuint.framework.web.PageResult;
import com.fuint.repository.mapper.MtPurchaseOrderItemMapper;
import com.fuint.repository.mapper.MtPurchaseOrderMapper;
import com.fuint.repository.mapper.MtStoreMapper;
import com.fuint.repository.model.MtPurchaseOrder;
import com.fuint.repository.model.MtPurchaseOrderItem;
import com.fuint.repository.model.MtStore;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;


/**
 * 采购订单表服务接口
 * Created by huangwen
 * CopyRight https://www.fuint.cn
 */
@Service
@AllArgsConstructor
public class PurchaseOrderServiceImpl extends ServiceImpl<MtPurchaseOrderMapper, MtPurchaseOrder> implements PurchaseOrderService {

	private static final Logger logger = LoggerFactory.getLogger(PurchaseOrderServiceImpl.class);

	private MtPurchaseOrderMapper mtPurchaseOrderMapper;

	private MtPurchaseOrderItemMapper mtPurchaseOrderItemMapper;

	private MtStoreMapper mtStoreMapper;

	private PurchaseOrderItemService purchaseOrderItemService;

	@Override
	public MtPurchaseOrderVo queryPurchaseOrderById(Long id) throws BusinessCheckException {

		Assert.isTrue(ObjectUtils.isNotEmpty(AuthUserUtil.get()), I18nUtil.getMessage("notAuthenticated"));
		Assert.isTrue(ObjectUtils.isNotEmpty(id), I18nUtil.getMessage("orderQuery"));

		MtPurchaseOrder mtPurchaseOrder = mtPurchaseOrderMapper.selectOne(new LambdaQueryWrapper<MtPurchaseOrder>().eq(MtPurchaseOrder::getId, id));
		if (Objects.isNull(mtPurchaseOrder)){
			throw new BusinessCheckException(I18nUtil.getMessage("orderMissing"));
		}
		List<MtPurchaseOrderItem> itemList = mtPurchaseOrderItemMapper.selectList(new LambdaQueryWrapper<MtPurchaseOrderItem>().eq(MtPurchaseOrderItem::getPurchaseNo, mtPurchaseOrder.getOrderNo()));

		MtPurchaseOrderVo mtPurchaseOrderVo = BeanUtil.copyProperties(mtPurchaseOrder, MtPurchaseOrderVo.class);
		if (StringUtils.isNotBlank(mtPurchaseOrderVo.getFileId())) {
			String[] split = mtPurchaseOrderVo.getFileId().split(",");
			List<String> fileList = Arrays.asList(split);
			mtPurchaseOrderVo.setFileList(fileList);
		}

		if (ObjectUtils.isNotEmpty(itemList)) {
			List<MtPurchaseOrderItemVo> mtPurchaseOrderItemVos = BeanUtil.copyToList(itemList, MtPurchaseOrderItemVo.class);
			mtPurchaseOrderVo.setOrderItemList(mtPurchaseOrderItemVos);
		}

		return mtPurchaseOrderVo;
	}


	@Override
	public PageResult<MtPurchaseOrderVo> queryOrderPageList(PurchaseOrderSearchDto mtPurchaseOrderDto) {

		AccountInfo accountInfo = AuthUserUtil.get();
		Assert.isTrue(ObjectUtils.isNotEmpty(accountInfo), I18nUtil.getMessage("notAuthenticated"));
		Long storeId = AuthUserUtil.get().getStoreId();
		//分页
		IPage<MtPurchaseOrder> page = new Page<>(mtPurchaseOrderDto.getPageNo() == null ? 1 : mtPurchaseOrderDto.getPageNo(), mtPurchaseOrderDto.getPageSize() == null ? 10 : mtPurchaseOrderDto.getPageSize());
		IPage<MtPurchaseOrder> orderIPage = mtPurchaseOrderMapper.selectPage(page, new LambdaQueryWrapper<MtPurchaseOrder>().
				eq((Objects.nonNull(storeId) && storeId > 0), MtPurchaseOrder::getStoreId, storeId));
		if (ObjectUtils.isEmpty(orderIPage.getRecords())) {
			return PageResult.<MtPurchaseOrderVo>builder().data(new ArrayList<>()).currentPage((int) orderIPage.getCurrent()).total((int) orderIPage.getTotal()).build();
		}
		List<MtPurchaseOrderVo> mtPurchaseOrderVos = BeanUtil.copyToList(orderIPage.getRecords(), MtPurchaseOrderVo.class);

		//订单项列表
       /* List<MtPurchaseOrderItem> itemList = mtPurchaseOrderItemMapper.selectList(null);
        Map<String, List<MtPurchaseOrderItem>> orderNoItemMap = itemList.stream().collect(Collectors.groupingBy(MtPurchaseOrderItem::getPurchaseNo));

        mtPurchaseOrderVos.forEach(vo -> {
            List<MtPurchaseOrderItem> orderItemList = orderNoItemMap.get(vo.getOrderNo());
            if (ObjectUtils.isNotEmpty(orderItemList)) {
                List<MtPurchaseOrderItemVo> orderItemVos = BeanUtil.copyToList(orderItemList, MtPurchaseOrderItemVo.class);
                vo.setOrderItemList(orderItemVos);
            }
        });*/
		return PageResult.<MtPurchaseOrderVo>builder().data(mtPurchaseOrderVos).total((int) orderIPage.getTotal()).build();
	}

	@Override
	@Transactional
	public void savePurcahseOrder(MtPurchaseOrderDto mtPurchaseOrderDto) throws BusinessCheckException {
		//参数验证
		AccountInfo accountInfo = AuthUserUtil.get();
		Assert.isTrue(ObjectUtils.isNotEmpty(accountInfo), I18nUtil.getMessage("notAuthenticated"));
		List<MtPurchaseOrderItemDto> orderItemList = mtPurchaseOrderDto.getOrderItemList();
		Assert.isFalse(orderItemList.isEmpty(), I18nUtil.getMessage("procurementItem"));
		Assert.isFalse(accountInfo.getStoreId() <= 0, I18nUtil.getMessage("superAdminNotAllowedUseMerchantAccount"));
		MtStore mtStore = mtStoreMapper.selectOne(new LambdaQueryWrapper<MtStore>().eq(MtStore::getId, accountInfo.getStoreId()).eq(MtStore::getStatus, StatusEnum.ENABLED.getKey()));
		Assert.isTrue(ObjectUtils.isNotEmpty(mtStore), I18nUtil.getMessage("novaDine"));
		String orderNo = mtPurchaseOrderDto.getOrderNo();
		Integer count = this.baseMapper.selectCount(new LambdaQueryWrapper<MtPurchaseOrder>()
				.eq(MtPurchaseOrder::getOrderNo, orderNo));
		if (count > 0) {
			throw new BusinessCheckException(I18nUtil.getMessage("ordNumEx"));
		}
		//参数准备
		MtPurchaseOrder purchaseOrder = BeanUtil.copyProperties(mtPurchaseOrderDto, MtPurchaseOrder.class);
		purchaseOrder.setCreateTime(new Date());
		purchaseOrder.setCreateUserId(String.valueOf(accountInfo.getId()));
		purchaseOrder.setOrderNo(orderNo != null ? orderNo : new Snowflake(1, 1).nextIdStr());

		List<MtPurchaseOrderItem> orderItems = new ArrayList<>();
		for (MtPurchaseOrderItemDto itemDto : orderItemList) {
			MtPurchaseOrderItem purchaseOrderItem = BeanUtil.copyProperties(itemDto, MtPurchaseOrderItem.class);
			purchaseOrderItem.setPurchaseNo(String.valueOf(purchaseOrder.getOrderNo()));
			purchaseOrderItem.setCreateUserId(String.valueOf(accountInfo.getId()));
			purchaseOrderItem.setTotalPrice(purchaseOrderItem.getUnitPrice().multiply(BigDecimal.valueOf(purchaseOrderItem.getTotalQuantity())));
			orderItems.add(purchaseOrderItem);
		}
		purchaseOrder.setTotalPrice(orderItems.stream().map(MtPurchaseOrderItem::getTotalPrice)
				.reduce(BigDecimal.ZERO, BigDecimal::add));
		purchaseOrder.setStoreId(accountInfo.getStoreId());
		purchaseOrder.setUserId(accountInfo.getId().toString());
		//保存订单
		int insert = mtPurchaseOrderMapper.insert(purchaseOrder);
		Assert.isFalse(insert != 1, I18nUtil.getMessage("saveErr"));
		//保存订单项
		boolean b = purchaseOrderItemService.saveBatch(orderItems);
		Assert.isTrue(b, I18nUtil.getMessage("saveErr"));
	}

	@Override
	@Transactional
	public void updatePurchaseOrderByOrderNo(MtPurchaseOrderDto mtPurchaseOrderDto) {
		//参数验证
		AccountInfo accountInfo = AuthUserUtil.get();
		Assert.isTrue(ObjectUtils.isNotEmpty(accountInfo), I18nUtil.getMessage("notAuthenticated"));
		Assert.isTrue(ObjectUtils.isNotEmpty(mtPurchaseOrderDto.getOrderNo()), I18nUtil.getMessage("pOSheet"));
		Assert.isFalse(mtPurchaseOrderDto.getOrderItemList().isEmpty(), I18nUtil.getMessage("procurementItem"));
		Assert.isFalse(accountInfo.getStoreId() == 0, I18nUtil.getMessage("superAdminNotAllowedUseMerchantAccount"));

		MtStore mtStore = mtStoreMapper.selectOne(new LambdaQueryWrapper<MtStore>().eq(MtStore::getId, accountInfo.getStoreId()).eq(MtStore::getStatus, StatusEnum.ENABLED.getKey()));
		Assert.isTrue(ObjectUtils.isNotEmpty(mtStore), I18nUtil.getMessage("phantomEats"));

		MtPurchaseOrder mtPurchaseOrder = mtPurchaseOrderMapper.selectOne(new LambdaQueryWrapper<MtPurchaseOrder>().eq(MtPurchaseOrder::getOrderNo, mtPurchaseOrderDto.getOrderNo()));
		Assert.isTrue(ObjectUtils.isNotEmpty(mtPurchaseOrder), I18nUtil.getMessage("orderMissing"));

		//修改订单
		MtPurchaseOrder purchaseOrder = BeanUtil.copyProperties(mtPurchaseOrderDto, MtPurchaseOrder.class);
		purchaseOrder.setTotalPrice(mtPurchaseOrder.getTotalPrice());
		purchaseOrder.setTotalQuantity(mtPurchaseOrder.getTotalQuantity());
		purchaseOrder.setTotalWeight(mtPurchaseOrder.getTotalWeight());
		purchaseOrder.setOrderNo(mtPurchaseOrder.getOrderNo());
		purchaseOrder.setId(mtPurchaseOrder.getId());
		purchaseOrder.setProject(mtPurchaseOrder.getProject());
		int update = mtPurchaseOrderMapper.updateById(purchaseOrder);
		Assert.isFalse(update != 1, I18nUtil.getMessage("updateErr"));

	}

	@Override
	@Transactional
	public void deletePurchaseOrderByOrderNo(Long id) {
		//参数验证
		AccountInfo accountInfo = AuthUserUtil.get();
		Assert.isTrue(ObjectUtils.isNotEmpty(accountInfo), I18nUtil.getMessage("notAuthenticated"));
		Assert.isTrue(ObjectUtils.isNotEmpty(id), I18nUtil.getMessage(I18nUtil.getMessage("pOSheet")));

		MtPurchaseOrder mtPurchaseOrder = mtPurchaseOrderMapper.selectOne(new LambdaQueryWrapper<MtPurchaseOrder>().eq(MtPurchaseOrder::getId, id));
		Assert.isTrue(ObjectUtils.isNotEmpty(mtPurchaseOrder), I18nUtil.getMessage("orderMissing"));
		Assert.isFalse(accountInfo.getStoreId() != 0 && mtPurchaseOrder.getStoreId() != accountInfo.getStoreId(), I18nUtil.getMessage("superAdminNotAllowedUseMerchantAccount"));

		int delete = mtPurchaseOrderMapper.deleteById(id);
		Assert.isFalse(delete != 1, I18nUtil.getMessage("deleteErr"));
		int delete1 = mtPurchaseOrderItemMapper.delete(new LambdaQueryWrapper<MtPurchaseOrderItem>().eq(MtPurchaseOrderItem::getPurchaseNo, mtPurchaseOrder.getOrderNo()));
		Assert.isFalse(delete1 != 1, I18nUtil.getMessage("deleteErr"));
	}
}
