package com.fuint.common.util;

import org.apache.poi.hssf.usermodel.*;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

/**
 * Excel工具
 *
 * Created by FSQ
 * CopyRight https://www.fuint.cn
 */
public class ExcelUtil {

    /**
     * 导出Excel
     * @param sheetName sheet名称
     * @param title 标题
     * @param values 内容
     * @param wb HSSFWorkbook对象
     * @return
     */
    public static HSSFWorkbook getHSSFWorkbook(String sheetName,String []title,String [][]values, HSSFWorkbook wb){

        // 第一步，创建一个HSSFWorkbook，对应一个Excel文件
        if(wb == null){
            wb = new HSSFWorkbook();
        }

        // 第二步，在workbook中添加一个sheet,对应Excel文件中的sheet
        HSSFSheet sheet = wb.createSheet(sheetName);

        // 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制
        HSSFRow row = sheet.createRow(0);

        // 第四步，创建单元格，并设置值表头 设置表头居中
        HSSFCellStyle style = wb.createCellStyle();
        style.setAlignment(HSSFCellStyle.ALIGN_CENTER); // 创建一个居中格式

        //声明列对象
        HSSFCell cell = null;

        //创建标题
        for(int i=0;i<title.length;i++){
            cell = row.createCell(i);
            cell.setCellValue(title[i]);
            cell.setCellStyle(style);
        }
        //创建内容
        if (values!=null) {
            //创建内容
            for (int i = 0; i < values.length; i++) {
                row = sheet.createRow(i + 1);
                for (int j = 0; j < values[i].length; j++) {
                    //将内容按顺序赋给对应的列对象
                    row.createCell(j).setCellValue(values[i][j]);
                }
            }
        }
        return wb;
    }

    /**
     * 发送响应流方法
     * @param  response
     * @param  fileName
     * @param  wb
     * */
    public static void setResponseHeader(HttpServletResponse response, String fileName, HSSFWorkbook wb) {
        try {
            // 编码文件名以适应RFC 5987标准
            String encodedFileName = URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20");

            // 设置响应内容类型（HSSFWorkbook对应.xls格式）
            response.setContentType("application/vnd.ms-excel");

            // 设置支持多浏览器UTF-8编码的文件名
            response.setHeader("Content-Disposition",
                    "attachment; filename=\"" + encodedFileName + "\"; filename*=UTF-8''" + encodedFileName);

            // 修正缓存控制头（修正拼写错误）
            response.setHeader("Pragma", "no-cache");
            response.setHeader("Cache-Control", "no-cache");
            response.setDateHeader("Expires", 0);
        } catch (UnsupportedEncodingException e) {
            // 处理编码异常（实际上UTF-8在Java标准中总是可用，此catch块为保险起见）
            response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
            e.printStackTrace();
        }

        // 使用try-with-resources确保流关闭（需要Java7+）
        try (OutputStream os = response.getOutputStream()) {
            wb.write(os);
            os.flush();
        } catch (Exception e) {
            // 记录异常并处理（实际生产环境应使用日志框架）
            e.printStackTrace();
            throw new RuntimeException("导出Excel失败", e);
        }
    }
}
