package com.fuint.module.backendApi.controller;

import com.fuint.common.dto.ext.AlertDTO;
import com.fuint.common.dto.ext.PageResult;
import com.fuint.common.dto.ext.SelectDto;
import com.fuint.common.dto.ext.ShopAlertRequest;
import com.fuint.common.service.StoreAlertService;
import com.fuint.common.vo.ShopDetailVO;
import com.fuint.common.vo.ShopVO;
import com.fuint.framework.exception.BusinessCheckException;
import com.fuint.framework.web.BaseController;
import com.fuint.framework.web.ResponseObject;
import com.fuint.repository.model.MtStore;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author Survive
 * @date 2025/3/12
 * @description
 */
@Api(tags="管理端-预警相关接口")
@RestController
@AllArgsConstructor
@RequestMapping("/backendApi/storeAlerts")
public class StoreAlertController extends BaseController {
    private StoreAlertService shopService;

    /**
     * 分页数据展示
     * @param page
     * @param size
     * @param name
     * @return
     */
    @GetMapping(value = "/list")
    
    @ApiOperation(value = "店铺列表")
    @PreAuthorize("@pms.hasPermission('store:alerts:list')")
    public PageResult<ShopVO> listShops(
            @RequestParam(defaultValue = "1") Integer page,
            @RequestParam(defaultValue = "10") Integer size,
            @RequestParam(required = false) String name,
            @RequestParam(required = false) String nextReminderDay
    ) throws BusinessCheckException{
	    return shopService.listShops(page, size, name,nextReminderDay);
    }

    /**
     * 店铺详情
     * @param shopId
     * @param withOrders 否携带当日订单
     * @return
     */
    @ApiOperation(value = "店铺详情")
    @GetMapping("/{shopId}")
    
    @PreAuthorize("@pms.hasPermission('store:alerts:list')")
//    @PreAuthorize("@pms.hasPermission('alerts:index')")
    public ResponseObject getShopDetail(HttpServletRequest request, @PathVariable Long shopId, @RequestParam(required = false) Boolean withOrders  ) throws BusinessCheckException {
        ShopDetailVO shopDetail = shopService.getShopDetail(shopId, withOrders);
        return getSuccessResult(shopDetail);
    }

    /**
     * 新增预警信息
     * @return
     */
    @ApiOperation(value = "新增预警")
    @RequestMapping(value = "/add", method = RequestMethod.POST)
    
    @PreAuthorize("@pms.hasPermission('store:alerts:add')")
//    @PreAuthorize("@pms.hasPermission('alerts:index')")
    public ResponseObject createAlert(@RequestBody ShopAlertRequest alertInfo) throws BusinessCheckException {
        shopService.addShopAlert(alertInfo);
        return getSuccessResult(true);
    }

    /**
     * 更新预警信息
     * @param shopId
     * @param request
     * @return
     */
    @ApiOperation(value = "更新预警")
    
    @PostMapping("/update/{shopId}")
    @PreAuthorize("@pms.hasPermission('store:alerts:edit')")
    //    @PreAuthorize("@pms.hasPermission('alerts:index')")
    public ResponseObject updateShopAlert(@PathVariable Long shopId, @RequestBody ShopAlertRequest request) throws BusinessCheckException{
        MtStore updatedShop = shopService.updateShopAlert(request);
        return getSuccessResult("更新成功",updatedShop);
    }

    /**
     * 清除预警信息
     * @param shopId
     * @return
     */
    @ApiOperation(value = "清除预警配置信息")
    
    @PostMapping("/clear/{shopId}")
    @PreAuthorize("@pms.hasPermission('store:alerts:edit')")
    //    @PreAuthorize("@pms.hasPermission('alerts:index')")
    public ResponseObject clearAlertConfig ( @PathVariable Long shopId ) throws BusinessCheckException{
        MtStore updatedShop = shopService.clearAlertConfig(shopId);
        return getSuccessResult("预警配置已清除", updatedShop);
    }

    @ApiOperation("获取预警区间")
    
    @GetMapping("/interval")
    @PreAuthorize("@pms.hasPermission('store:alerts:list')")
    public ResponseObject alertInterval(){
        List<AlertDTO> result = shopService.alertInterval();
        return getSuccessResult(result);
    }

    @ApiOperation("获取可选择的预警店铺")
    
    @GetMapping("/earlyWarningStore")
    @PreAuthorize("@pms.hasPermission('store:alerts:list')")
    public ResponseObject earlyWarningStore(@RequestParam(required = false) String storeName){
        List<SelectDto> result = shopService.earlyWarningStore(storeName);
        return getSuccessResult(result);
    }
}
