package com.fuint.repository.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.fuint.repository.bean.RevenueResult;
import com.fuint.repository.bean.StorePayAmountBean;
import com.fuint.repository.model.MtOrder;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 订单表 Mapper 接口
 *
 * Created by FSQ
 * CopyRight https://www.fuint.cn
 */
public interface MtOrderMapper extends BaseMapper<MtOrder> {

    BigDecimal getOrderCount(@Param("merchantId") Long merchantId);

    BigDecimal getStoreOrderCount(@Param("storeId") Long storeId);

    BigDecimal getOrderCountByTime(@Param("merchantId") Long merchantId, @Param("beginTime") Date beginTime, @Param("endTime") Date endTime);

    BigDecimal getStoreOrderCountByTime(@Param("storeId") Long storeId, @Param("beginTime") Date beginTime, @Param("endTime") Date endTime);

    MtOrder findByOrderSn(@Param("orderSn") String orderSn);

    MtOrder findByTableId(@Param("tableId") Long tableId);

    BigDecimal getPayMoney(@Param("merchantId") Long merchantId);

    BigDecimal getPayMoneyByTime(@Param("merchantId") Long merchantId, @Param("beginTime") Date beginTime, @Param("endTime") Date endTime);

    BigDecimal getStorePayMoneyByTime(@Param("storeId") Long storeId, @Param("beginTime") Date beginTime, @Param("endTime") Date endTime);

    BigDecimal getStorePayMoney(@Param("storeId") Long storeId);

    Integer getPayUserCount(@Param("merchantId") Long merchantId);

    Integer getStorePayUserCount(@Param("storeId") Long storeId);

    BigDecimal getUserPayMoney(@Param("userId") Long userId);

    Integer getUserPayOrderCount(@Param("userId") Long userId);

    List<MtOrder> getTobeCommissionOrderList(@Param("dateTime") String dateTime);

    @Select("SELECT " +
            "  s.name AS storeName, " +
            "  YEAR(o.pay_time) AS year, " +
            "  COALESCE(SUM(o.amount), 0) AS totalAmount " +
            "FROM mt_store s " +
            "LEFT JOIN mt_order o ON s.id = o.store_id " +
            "  AND YEAR(o.pay_time) BETWEEN #{startYear} AND #{endYear} " +
            "GROUP BY s.id, YEAR(o.pay_time) " +
                    "ORDER BY year")
    List<RevenueResult> selectYearlyRevenue(
            @Param("startYear") Integer startYear,
            @Param("endYear") Integer endYear
    );

	List<StorePayAmountBean> revenueStatistics(@Param("startTime") String startTime,
                                               @Param("endTime") String endTime,
                                               @Param("storeId") Long storeId);

    List<MtOrder> refundOrder(@Param("storeId") Long storeId, @Param("beginTime") Date beginTime, @Param("endTime") Date endTime);

    List<MtOrder> getRefundOrderyByMerchantId(@Param("merchantId") Long merchantId, @Param("beginTime") Date beginTime, @Param("endTime") Date endTime);

    List<MtOrder> getTransactionsList(@Param("storeIds") List<Long> storeIds,
                                      @Param("beginTime") String startTime,
                                      @Param("endTime") String endTime);
}
